unit Unit2;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.ExtCtrls, Vcl.AppEvnts,
  System.ImageList, Vcl.ImgList, UUSBRelayDev;


const
  mask:array[0..7] of byte=(1,2,4,8,16,32,64,128);

type
  TForm2 = class(TForm)
    Memo1: TMemo;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Button4: TButton;
    Button5: TButton;
    Button6: TButton;
    Button7: TButton;
    Button9: TButton;
    Label1: TLabel;
    Label2: TLabel;
    ApplicationEvents1: TApplicationEvents;
    ImageList1: TImageList;
    pb1: TPaintBox;
    pb2: TPaintBox;
    ComboBox1: TComboBox;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure Button9Click(Sender: TObject);
    procedure ApplicationEvents1Idle(Sender: TObject; var Done: Boolean);
    procedure pb1Paint(Sender: TObject);
    procedure pb2Paint(Sender: TObject);
    procedure Button6Click(Sender: TObject);
  private
    rhandle: intptr;                      //handle to the relay device
    idstr: p_usb_relay_device_info;       //devive ID string
    procedure SetLamps;
  public
    { Public declarations }
  end;

var
  Form2: TForm2;

implementation

{$R *.dfm}

Function BitTest(a,bit:byte):Boolean;
begin
  BitTest:=a and mask[bit] <> 0;
end;

procedure TForm2.ApplicationEvents1Idle(Sender: TObject; var Done: Boolean);
begin
  button1.Enabled := rhandle = 0;
  button9.Enabled := not button1.Enabled;
end;

procedure TForm2.Button1Click(Sender: TObject);
var
  i: integer;
begin
  if rhandle = 0 then
  begin
    rhandle := usb_relay_device_open_with_serial_number(pansichar(idstr.idstr),5); //HURTM serial for 2 relay board   //
    memo1.Lines.Add('Device Handle: ' + inttostr(rhandle));
    i := usb_relay_device_lib_version();
    memo1.Lines.Add('DLL Version: ' + inttostr(i));
    memo1.Lines.Add('Channels: ' + inttostr(usb_relay_device_get_num_relays(idstr)));
  end;
  setlamps;
end;

procedure TForm2.Button2Click(Sender: TObject);
var
  i: integer;
begin
  if rhandle <> 0 then
  begin
    i := usb_relay_device_open_all_relay_channel(rhandle);
  end
  else
    rhandle := 0;
  setlamps;
end;

procedure TForm2.Button3Click(Sender: TObject);
var
  i: integer;
begin
  if rhandle <> 0 then
  begin
    i := usb_relay_device_close_all_relay_channel(rhandle);
  end
  else
    rhandle := 0;
  setlamps;
end;

procedure TForm2.Button4Click(Sender: TObject);
begin
  if rhandle <> 0 then
    usb_relay_device_open_one_relay_channel(rhandle, tbutton(sender).Tag);
  setlamps;
end;

procedure TForm2.Button6Click(Sender: TObject);
begin
  if rhandle <> 0 then
    usb_relay_device_close_one_relay_channel(rhandle, tbutton(sender).Tag);
  setlamps;
end;

procedure TForm2.Button9Click(Sender: TObject);
begin
  if rhandle <> 0 then
  begin
    usb_relay_device_close_all_relay_channel(rhandle);
    usb_relay_device_close(rhandle);
    rhandle := 0;
    memo1.Text := 'Device ID: ' + idstr.idstr;
  end;
  setlamps;
end;

procedure TForm2.FormClose(Sender: TObject; var Action: TCloseAction);
var
  i: integer;
begin
  if rhandle <> 0 then
  begin
    usb_relay_device_close_all_relay_channel(rhandle);
    setlamps;
    usb_relay_device_close(rhandle);
    usb_relay_device_free_enumerate(idstr);
  end;
  i := usb_relay_exit();
end;

procedure TForm2.FormCreate(Sender: TObject);
var
  i: integer;
begin
  rhandle := 0;
  i := usb_relay_init();
  if i <> 0 then
  begin
    application.MessageBox('DLL initialise failed!','ERROR',mb_setforeground or mb_iconexclamation);
    exit;
  end;
  setlamps;
  combobox1.Clear;
  idstr := usb_relay_device_enumerate();
  {usb_relay_device_next_dev() usage
   To add to a list would need an object to contain a p_usb_relay_device_info
   I am not convinced it is possible to add multiple devices as they all seem
   to use the same ID string!
  }
  {combobox1.AddItem(idstr.idstr,nil); //replace nil with a Delphi object
  while idstr <> nil do
  begin
    idstr := usb_relay_device_next_dev(idstr);
    combobox1.AddItem(idstr.idstr,nil);
  end;}
  memo1.Lines.Add('Device ID: ' + idstr.idstr);
end;

procedure TForm2.pb1Paint(Sender: TObject);
var
  bmp: tbitmap;
  status: cardinal;
begin
  bmp := tbitmap.Create;
  try
    if rhandle <> 0 then
    begin
      usb_relay_device_get_status(rhandle, status);
      if status in [1,3] then
        imagelist1.GetBitmap(1,bmp)
      else
        imagelist1.GetBitmap(0,bmp);
      pb1.Canvas.Draw(0,0,bmp);
    end
    else begin
      imagelist1.GetBitmap(0,bmp);
      pb1.Canvas.Draw(0,0,bmp);
    end;
  finally
    bmp.Free;
  end;
end;

procedure TForm2.pb2Paint(Sender: TObject);
var
  bmp: tbitmap;
  status: cardinal;
begin
  bmp := tbitmap.Create;
  try
    if rhandle <> 0 then
    begin
      usb_relay_device_get_status(rhandle, status);
      if status in [2,3] then
        imagelist1.GetBitmap(1,bmp)
      else
        imagelist1.GetBitmap(0,bmp);
      pb2.Canvas.Draw(0,0,bmp);
    end
    else begin
      imagelist1.GetBitmap(0,bmp);
      pb2.Canvas.Draw(0,0,bmp);
    end;
  finally
    bmp.Free;
  end;
end;

procedure TForm2.SetLamps;
var
  rnum: integer;
  b, i: byte;
begin
  pb1.Invalidate;
  pb2.Invalidate;
  if rhandle <> 0 then
  begin
    memo1.Clear;
    rnum := usb_relay_device_get_num_relays(idstr)-1; //relay count
    b := usb_relay_device_get_status_bitmap(rhandle); //test them all
    for i := 0 to rnum do
    begin
      if BitTest(b,i) then
        memo1.Lines.Add('Relay ' + inttostr(i + 1) + ' on.')
      else
        memo1.Lines.Add('Relay ' + inttostr(i + 1) + ' off.');
    end;
  end;
end;

end.
